## AdaBoost: overly simplified implementation of AdaBoost with decision stumps
##
## Author: Vlad Popovici
##         Bioinformatics Core Facility
##         Swiss Institute of Bioinformatics

require(rpart)

## Data matrices Xtr, Xts must have the samples by rows! 
## The label vectors ytr and yts must have values +/-1.
simple.adaboost = function(Xtr, ytr, Xts, yts, iterations=100)
{
  f = 0
  f.ts = 0
  n.tr = length(ytr)
  n.ts = length(yts)
  tr.err = rep(0, iterations)
  ts.err = rep(0, iterations)
  i = 1
  while (i <= iterations) {
    w = exp(-ytr*f)
    w = w/sum(w)

    stump = rpart(y ~. , data.frame(Xtr, y=ytr), w, method="class",
      control=rpart.control(minsplit=0,minbucket=0,cp=-1,maxcompete=0, 
          maxsurrogate=0, usesurrogate=0, xval=0, maxdepth=1))
    # predictions on training data:
    p  = predict(stump, data.frame(Xtr), type="prob") # posteriors 
    p1 = p[,1] # P(c1 | x)
    p2 = p[,2] # P(c2 | x)
    gmin.tr = 2 * (p1 < p2) - 1
    # predictions on validation data:
    p  = predict(stump, data.frame(Xts), type="prob") # posteriors 
    p1 = p[,1] # P(c1 | x)
    p2 = p[,2] # P(c2 | x)
    gmin.ts = 2 * (p1 < p2) - 1

    e = sum( w*(1 - (ytr == gmin.tr)) )
    alpha = 0.5*log ( (1-e) / e )

    f    = f + alpha*gmin.tr
    f.ts = f.ts + alpha*gmin.ts

    tr.err[i] = sum(f*ytr < 0) / n.tr
    ts.err[i] = sum(f.ts*yts < 0) / n.ts
        
    i = i + 1
  }
  
  return (list(tr.err, ts.err))
}

